<?php

namespace Drupal\phylotree_viewer\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class PhyloTreeViewerSettingsForm.
 *
 * @package Drupal\phylotree_viewer\Form
 */
class PhyloTreeViewerSettingsForm extends ConfigFormBase {

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  public function __construct(ConfigFactoryInterface $config_factory,
                              MessengerInterface $messenger,
                              ModuleHandlerInterface $module_handler) {
    parent::__construct($config_factory);
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('config.factory'),
          $container->get('messenger'),
          $container->get('module_handler')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'phylotree_viewer_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'phylotree_viewer.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('phylotree_viewer.settings');

    $form['test_libraries'] = [
      '#prefix' => '<div>&nbsp;</div>',
      '#type' => 'submit',
      '#value' => 'Test libraries',
      '#submit' => ['::testLibraries'],
    ];

    $allow_all_extensions = $config->get('allow_all_extensions');
    $form['allow_all_extensions'] = [
      '#type' => 'checkbox',
      '#description' => "Check this if you want to render all files with Phylotree Viewer.",
      '#title' => 'Allow all extensions',
      '#default_value' => $allow_all_extensions,
    ];

    $supported_extensions = $config->get('supported_extensions');
    $form['supported_extensions'] = [
      '#title' => 'Supported Extensions',
      '#type' => 'textarea',
      '#default_value' => $supported_extensions,
      '#states' => [
        'visible' => [
          ':input[name=allow_all_extensions]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $form['reset'] = [
      '#type' => 'submit',
      '#value' => 'Restore default supported extensions',
      '#submit' => ['::resetSubmit'],
    ];

    // Use the module handler service to get the module path.
    $modulePath = $this->moduleHandler->getModule('phylotree_viewer')->getPath();

    $phylotree_css = $modulePath . '/libraries/css/phylotree.css';
    $d3_js = $modulePath . '/libraries/js/d3.min.js';
    $jquery_js = $modulePath . '/libraries/js/jquery.min.js';
    $phylotree_js = $modulePath . '/libraries/js/phylotree.js';
    $underscore_js = $modulePath . '/libraries/js/underscore.min.js';

    if (!file_exists($phylotree_css)) {
      $this->messenger->addError($phylotree_css . ' does not exist.');
    }

    if (!file_exists($d3_js)) {
      $this->messenger->addError($d3_js . ' does not exist.');
    }

    if (!file_exists($jquery_js)) {
      $this->messenger->addError($jquery_js . ' does not exist.');
    }

    if (!file_exists($phylotree_js)) {
      $this->messenger->addError($phylotree_js . ' does not exist.');
    }

    if (!file_exists($underscore_js)) {
      $this->messenger->addError($underscore_js . ' does not exist.');
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function testLibraries(array &$form, FormStateInterface $form_state): void {
    // Use the module handler service to get the module path.
    $modulePath = $this->moduleHandler->getModule('phylotree_viewer')->getPath();

    $phylotree_css = $modulePath . '/libraries/css/phylotree.css';
    $d3_js = $modulePath . '/libraries/js/d3.min.js';
    $jquery_js = $modulePath . '/libraries/js/jquery.min.js';
    $phylotree_js = $modulePath . '/libraries/js/phylotree.js';
    $underscore_js = $modulePath . '/libraries/js/underscore.min.js';

    if (file_exists($phylotree_css)) {
      $this->messenger->addMessage($phylotree_css . ' exists.');
    }
    else {
      $this->messenger->addError($phylotree_css . ' does not exist.');
    }

    if (file_exists($d3_js)) {
      $this->messenger->addMessage($d3_js . ' exists.');
    }
    else {
      $this->messenger->addError($d3_js . ' does not exist.');
    }

    if (file_exists($jquery_js)) {
      $this->messenger->addMessage($jquery_js . ' exists.');
    }
    else {
      $this->messenger->addError($jquery_js . ' does not exist.');
    }

    if (file_exists($phylotree_js)) {
      $this->messenger->addMessage($phylotree_js . ' exists.');
    }
    else {
      $this->messenger->addError($phylotree_js . ' does not exist.');
    }

    if (file_exists($underscore_js)) {
      $this->messenger->addMessage($underscore_js . ' exists.');
    }
    else {
      $this->messenger->addError($underscore_js . ' does not exist.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function resetSubmit(array &$form, FormStateInterface $form_state) {
    $this->config('phylotree_viewer.settings')
      ->set('supported_extensions', $this->config('phylotree_viewer.settings.default')->get('supported_extensions'))
      ->set('allow_all_extensions', $this->config('phylotree_viewer.settings.default')->get('allow_all_extensions'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('phylotree_viewer.settings')
      ->set('supported_extensions', $form_state->getValue('supported_extensions'))
      ->set('allow_all_extensions', $form_state->getValue('allow_all_extensions'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
