<?php

namespace Drupal\phylotree_viewer\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Plugin\Field\FieldFormatter\FileFormatterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'phylotree_viewer' formatter.
 *
 * @FieldFormatter(
 *   id = "phylotree_viewer",
 *   label = @Translation("Phylotree viewer"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class PhyloTreeViewer extends FileFormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The File System service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a PDBFieldViewer instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the viewer.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the viewer is associated.
   * @param array $settings
   *   The viewer settings.
   * @param string $label
   *   The viewer label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration interface.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   */
  public function __construct($plugin_id,
                              $plugin_definition,
                              FieldDefinitionInterface $field_definition,
                              array $settings,
                              $label,
                              $view_mode,
                              array $third_party_settings,
                              ConfigFactoryInterface $configFactory,
                              Messenger $messenger,
                              FileSystemInterface $fileSystem,
                              FileUrlGeneratorInterface $file_url_generator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->fileSystem = $fileSystem;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $plugin_id,
          $plugin_definition,
          $configuration['field_definition'],
          $configuration['settings'],
          $configuration['label'],
          $configuration['view_mode'],
          $configuration['third_party_settings'],
          $container->get('config.factory'),
          $container->get('messenger'),
          $container->get('file_system'),
          $container->get('file_url_generator')
      );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'show_file_link' => 0,
      'show_file_size' => 0,
      'max_file_size' => 0,
      'return_empty' => 0,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['show_file_link'] = [
      '#title' => $this->t("Show the file's name with a link to the file."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('show_file_link'),
    ];

    $form['show_file_size'] = [
      '#title' => $this->t("Show the file's size."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('show_file_size'),
    ];

    $form['max_file_size'] = [
      '#title' => $this->t('Max file size'),
      '#description' => $this->t('Limit this file formatter to a maximum file size in bytes. 0 indicates unlimited.'),
      '#type' => 'textfield',
      '#attributes' => [
        ' type' => 'number',
      ],
      '#element_validate' => [[$this, 'validateSize']],
      '#required' => TRUE,
      '#default_value' => $this->getSetting('max_file_size'),
    ];

    $form['return_empty'] = [
      '#title' => $this->t('Disable all content if file format not recognized.'),
      '#description' => $this->t("If a file's content is not recognized and cannot be drawn, show nothing."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('return_empty'),
    ];

    return $form;
  }

  /**
   * Validates width form element.
   *
   * @param array $element
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $complete_form
   *   The complete form structure.
   */
  public static function validateSize(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    $size = $element['#value'];

    if (!is_numeric($size) || $size < 0) {
      $form_state->setErrorByName("max_file_size", "The max file size for PhyloTreeViewer must be greater than or equal to 0.");
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();
    $summary[] = "Display tree.";
    if ($settings['return_empty'] == 1) {
      $summary[] = $this->t('Empty if file type is unrecognized.');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $settings = $this->getSettings();

    // Create an empty render array.
    $elements = [];

    $config = $this->configFactory->get('phylotree_viewer.settings');

    $allow_all_extension = $config->get("allow_all_extensions");
    $supported_extensions = explode(" ", $config->get('supported_extensions'));

    // Loop through the entities we need to process and create a render
    // array for each one.
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {
      $markup = '';
      $load_libraries = FALSE;

      // Get the server file system's real path to the file.
      /** @var \Drupal\file\FileInterface $file */
      $path = $this->fileSystem->realpath($file->getFileUri());
      if ($path === FALSE) {
        $markup = '<p>' . $this->t('A server error occurred. The file could not be found.') . '</p>';
        if ($settings['return_empty'] == 1) {
          return [];
        }
      }

      // Get the user-visible file name.
      $filename = $file->getFilename();
      // $filepath = file_create_url($file->getFileUri());
      $filepath = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());

      // Get the file name extension and see if it is supported by
      // any of the known file formats.
      $ext = pathinfo($filename, PATHINFO_EXTENSION);

      $content = "";
      if ($allow_all_extension || ($ext && in_array($ext, $supported_extensions)) && $markup === '') {
        if ($settings['max_file_size'] > 0) {
          if ($file->getSize() > $settings['max_file_size']) {
            $markup .= '<a href="' . $filepath . '">' . $filename . '</a>';
            $this->messenger->addError('Unable to render file with PhyloTreeViewer as file exceeds ' . $settings['max_file_size'] . ' bytes.');
          }
        }

        if ($markup === '') {
          if ($settings['show_file_link'] == 1) {
            if ($settings['show_file_size'] == 1) {
              $markup .= '<p><a href="' . $filepath . '">' . $filename . '</a> (' . $file->getSize() . ' bytes)</p>';
            }
            else {
              $markup .= '<p></p><a href="' . $filepath . '">' . $filename . '</a></p>';
            }
          }
          else {
            if ($settings['show_file_size'] == 1) {
              $markup .= '<p>File Size: ' . $file->getSize() . ' bytes</p>';
            }
          }

          $markup .= $this->getPhlyotreeMarkup();
          $load_libraries = TRUE;

          $content = file_get_contents($file->getFileUri());
        }
      }

      if ($markup === '' && (!in_array($ext, $supported_extensions) || !$ext)) {
        $markup = '<a href="' . $filepath . '">' . $filename . '</a>';
        if ($settings['return_empty'] == 1) {
          return [];
        }
      }

      if ($load_libraries) {
        $library = 'phylotree_viewer/phylotree_viewer.local';

        $elements[$delta] = [
          '#theme' => 'phylotree_viewer',
          '#theme_markup' => $markup,
          '#attached' => [
            'library' => [
              $library,
            ],
            'drupalSettings' => [
              'files' => [
                'contents' => $content,
              ],
            ],
          ],
        ];
      }
      else {
        $elements[$delta] = [
          '#theme' => 'phylotree_viewer',
          '#theme_markup' => $markup,
        ];
      }
    }

    return $elements;
  }

  /**
   * Get the HTML for phlyotree.
   */
  private function getPhlyotreeMarkup(): string {
    return '<div class = \'container\' id = "main_display">
                    <div class = \'row\'>
                        <div class = \'col-md-12\'>
                        
                            <div class="btn-toolbar" role="toolbar">
                              <div class="btn-group">
                                <button type="button" class="btn btn-default btn-sm" data-direction = \'vertical\' data-amount = \'1\' title = "Expand vertical spacing">
                                    <i class="fa fa-arrows-v" ></i>
                                </button>
                                 <button type="button" class="btn btn-default btn-sm" data-direction = \'vertical\' data-amount = \'-1\' title = "Compress vertical spacing">
                                    <i class="fa fa-compress fa-rotate-135" ></i>
                                </button>
                                <button type="button" class="btn btn-default btn-sm" data-direction = \'horizontal\' data-amount = \'1\' title = "Expand horizonal spacing">
                                    <i class="fa fa-arrows-h" ></i>
                                </button>
                                 <button type="button" class="btn btn-default btn-sm" data-direction = \'horizontal\' data-amount = \'-1\' title = "Compress horizonal spacing">
                                    <i class="fa fa-compress fa-rotate-45" ></i>
                                </button>
                                 <button type="button" class="btn btn-default btn-sm" id = "sort_ascending" title = "Sort deepest clades to the bototm">
                                    <i class="fa fa-sort-amount-asc" ></i>
                                </button>
                                 <button type="button" class="btn btn-default btn-sm" id = "sort_descending" title = "Sort deepsest clades to the top">
                                    <i class="fa fa-sort-amount-desc" ></i>
                                </button>
                                 <button type="button" class="btn btn-default btn-sm" id = "sort_original" title = "Restore original order">
                                    <i class="fa fa-sort" ></i>
                                </button>
                              </div>
                              
                            <div class="btn-group" data-toggle="buttons">
                              <label class="btn btn-default active btn-sm">
                                <input type="radio" name="options" class = "phylotree-layout-mode" data-mode = "linear" autocomplete="off" checked title = "Layout left-to-right"> Linear
                              </label>
                              <label class="btn btn-default  btn-sm">
                                <input type="radio" name="options" class = "phylotree-layout-mode" data-mode = "radial" autocomplete="off" title = "Layout radially"> Radial
                              </label>
                            </div>
                            
                            <div class="btn-group">
                                <button type="button" class="btn btn-default btn-sm active" id = "toggle_animation" title = "Toggle animation">
                                    Animation
                                </button>
                              </label>
                            </div>
                
                            <div class="btn-group" data-toggle="buttons">
                              <label class="btn btn-default active btn-sm">
                                <input type="radio" class = "phylotree-align-toggler" data-align = "left" name="options-align" autocomplete="off" checked title = "Align tips labels to branches">
                                    <i class="fa fa-align-left" ></i>
                                </input>
                
                              </label>
                              <label class="btn btn-default btn-sm">
                               <input type="radio" class = "phylotree-align-toggler" data-align = "right" name="options-align" autocomplete="off" title = "Align tips labels to the edge of the plot">
                                    <i class="fa fa-align-right" ></i>
                                </input>
                              </label>
                            </div>
                
                           </div>
                        </div>
                    </div>
                
                    <div class = \'row\'>
                        <div class = \'col-md-12\'>
                            <div id = \'tree_container\' class = \'tree-widget\'>
                            </div>
                        </div>
                    </div>
                </div>';
  }

}
